@@ -37,38 +37,58 @@ namespace Kerfuffle
     class Query;
 }
 
+/**
+ * Meta data related to one entry in a compressed archive.
+ *
+ * This is used for indexing entry properties as numbers
+ * and for determining data displaying order in part's view.
+ */
+enum EntryMetaDataType {
+    FullPath,            /**< The entry's file name */
+    Size,                /**< The entry's original size */
+    CompressedSize,      /**< The compressed size for the entry */
+    Permissions,         /**< The entry's permissions */
+    Owner,               /**< The user the entry belongs to */
+    Group,               /**< The user group the entry belongs to */
+    Ratio,               /**< The compression ratio for the entry */
+    CRC,                 /**< The entry's CRC */
+    Method,              /**< The compression method used on the entry */
+    Version,             /**< The archiver version needed to extract the entry */
+    Timestamp            /**< The timestamp for the current entry */
+};
+
 class ArchiveModel: public QAbstractItemModel
 {
     Q_OBJECT
 public:
-    explicit ArchiveModel(const QString &dbusPathName, QObject *parent = 0);
-    ~ArchiveModel();
+    explicit ArchiveModel(const QString &dbusPathName, QObject *parent = nullptr);
+    ~ArchiveModel() override;
 
-    QVariant data(const QModelIndex &index, int role) const Q_DECL_OVERRIDE;
-    Qt::ItemFlags flags(const QModelIndex &index) const Q_DECL_OVERRIDE;
+    QVariant data(const QModelIndex &index, int role) const override;
+    Qt::ItemFlags flags(const QModelIndex &index) const override;
     QVariant headerData(int section, Qt::Orientation orientation,
-                        int role = Qt::DisplayRole) const Q_DECL_OVERRIDE;
+                        int role = Qt::DisplayRole) const override;
     QModelIndex index(int row, int column,
-                      const QModelIndex &parent = QModelIndex()) const Q_DECL_OVERRIDE;
-    QModelIndex parent(const QModelIndex &index) const Q_DECL_OVERRIDE;
-    int rowCount(const QModelIndex &parent = QModelIndex()) const Q_DECL_OVERRIDE;
-    int columnCount(const QModelIndex &parent = QModelIndex()) const Q_DECL_OVERRIDE;
-
-    virtual void sort(int column, Qt::SortOrder order = Qt::AscendingOrder) Q_DECL_OVERRIDE;
+                      const QModelIndex &parent = QModelIndex()) const override;
+    QModelIndex parent(const QModelIndex &index) const override;
+    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
+    int columnCount(const QModelIndex &parent = QModelIndex()) const override;
 
     //drag and drop related
-    Qt::DropActions supportedDropActions() const Q_DECL_OVERRIDE;
-    QStringList mimeTypes() const Q_DECL_OVERRIDE;
-    QMimeData *mimeData(const QModelIndexList & indexes) const Q_DECL_OVERRIDE;
-    bool dropMimeData(const QMimeData * data, Qt::DropAction action, int row, int column, const QModelIndex & parent) Q_DECL_OVERRIDE;
+    Qt::DropActions supportedDropActions() const override;
+    QStringList mimeTypes() const override;
+    QMimeData *mimeData(const QModelIndexList & indexes) const override;
+    bool dropMimeData(const QMimeData * data, Qt::DropAction action, int row, int column, const QModelIndex & parent) override;
 
     void reset();
     void createEmptyArchive(const QString &path, const QString &mimeType, QObject *parent);
     KJob* loadArchive(const QString &path, const QString &mimeType, QObject *parent);
     Kerfuffle::Archive *archive() const;
 
+    QList<int> shownColumns() const;
+    QMap<int, QByteArray> propertiesMap() const;
+
     Archive::Entry *entryForIndex(const QModelIndex &index);
-    int childCount(const QModelIndex &index, int &dirs, int &files) const;
 
     Kerfuffle::ExtractJob* extractFile(Archive::Entry *file, const QString& destinationDir, const Kerfuffle::ExtractionOptions& options = Kerfuffle::ExtractionOptions()) const;
     Kerfuffle::ExtractJob* extractFiles(const QVector<Archive::Entry*>& files, const QString& destinationDir, const Kerfuffle::ExtractionOptions& options = Kerfuffle::ExtractionOptions()) const;
